/*-----------------------------------------------------------------------------
  name:         nodemcu_platform_temp
  description:  platform remote temperature and humidity
  Version:      2017.07.19
  By:           Tim Mikkelsen
  Created:      07/05/2017
  Updated:      07/19/2017

  This is a more platform oriented embedded system initally to be
  used for a temperature/humidity sensor system (DHT11).  The
  platform portion is to allow for switching between 2 networks
  (home and cabin) via a switch and to allow for a 3 bit address
  for adding to a base IP address and port number.

  Hardware:
  - HiLetgo New Version NodeMCU LUA WiFi Internet ESP8266
  - DHT11 temperature and humidity sensor
  - USB power supply

  Additional libraries and code segments:
  - base wifi server code (by dlf modified by Tom Igoe)
  - Adafruit sensor and DHT libraries
  - example DHT code from Rui Santos

  To make the device work outside of your local network, enable
  your router to port forwarding using your external WAN address with
  the ThingPort value.  Setting the router varies by brand and will
  require looking at the router information.  Once set, an example is:
  - for a   local address   of   192.168.0.110 thru 117
  - for a   NodeMCU         of   8190 thru 8197
  - for a   WAN address     of   50.170.100.101
  -----------------------------------------------------------
  - you would read status   at   http://50.170.100.101:8190 (thru 8197)

*/


// ----------------------------------------------------------------------------
// libraries, definitions, devices, constants, variables
// ----------------------------------------------------------------------------

// libraries  -----------------------------------------------------------------
#include <ESP8266WiFi.h>
#include <DHT.h>

// device constants  ----------------------------------------------------------
#define LED          D0                 // LED pin -    GPIO16 = D0
#define DHTPin       D1                 // DHT pin -    GPIO5  = D1
#define WhichNetwork D2                 // switch pin - GPIO4  = D2
#define ipBit0       D5                 // switch pin - GPIO14 = D5
#define ipBit1       D6                 // switch pin - GPIO12 = D6
#define ipBit2       D7                 // switch pin - GPIO13 = D7



#define DHTTYPE  DHT11                  // DHT 11 sensor

// code constants  ------------------------------------------------------------
const String codeVersion = "NodeMCU Platform Temp 2017.07.19";


// variables  -----------------------------------------------------------------
static char celsiusTmpStr[7];
static char fahrenheitTmpStr[7];
static char humidityStr[7];
float h, t, f, hic, hif;

// WiFi and internet variables and constants ----------------------------------
int myNetwork;
int myIPaddr;                           // IP modifier
char ssid[] = "SSID";                   // your network SSID (name)
char pass[] = "password";               // your network password

int WifiStatus = WL_IDLE_STATUS;
int IoTPort;                            // port number to allow remote access
// check http://mobinnet.ir/uploaded/common_ports.pdf for unused port

// create devices  ------------------------------------------------------------
WiFiServer server(0);                   // create dummy wifi server with null port
DHT dht(DHTPin, DHTTYPE);

// ----------------------------------------------------------------------------
// Arduino setup  -------------------------------------------------------------
// ----------------------------------------------------------------------------
void setup() {

  // initialize Serial and wait for port to open:  ----------------------------
  // for some reason it appears I have to do some serial/debug output for this to work
  Serial.begin(9600);                   // for debug change '// Serial.' with 'Serial.'

  // initialize I/O  ----------------------------------------------------------
  pinMode(LED, OUTPUT);                 // on board LED
  pinMode(ipBit0, INPUT_PULLUP);        // bit 0 of IP address modifier
  pinMode(ipBit1, INPUT_PULLUP);        // bit 1 of IP address modifier
  pinMode(ipBit2, INPUT_PULLUP);        // bit 2 of IP address modifier
  pinMode(WhichNetwork, INPUT_PULLUP);  // network selector

  // initialize I/O  ----------------------------------------------------------
  dht.begin();                          // start sensor library

  // use for wifi debug output on Serial monitor
  // Serial.setDebugOutput(true);

  // check for the presence of WiFi  ------------------------------------------
  if (WiFi.status() == WL_NO_SHIELD) {
    // Serial.println("WIFI: Hardware not present");
    // don't continue:
    while (true);
  }

  WiFi.mode(WIFI_STA);                                      // make sure Wifi is not an access point

  // get network and IP modifier
  myNetwork = digitalRead(WhichNetwork);                    // read the network switch
  myIPaddr = digitalRead(ipBit0) +                          // calculate the address modifier
             digitalRead(ipBit1) * 2 +
             digitalRead(ipBit2) * 4;

  IoTPort = 8190 + myIPaddr;                                // port number to allow remote access
  WiFiServer server(IoTPort);                               // create wifi server with port

  if ( myNetwork == 0 ) {                                   // cabin network
    IPAddress myIP(192, 168, 0, 110 + myIPaddr);            // your desired IP address
    IPAddress gateway(192, 168, 0, 1);                      // your network gateway address
    IPAddress subnet(255, 255, 255, 0);                     // subnet mask
    // attempt to connect to Wifi network:                  // only try if not yet connected
    if ( WiFi.status() != WL_CONNECTED) {
      WiFi.config(myIP, gateway, subnet);                   // Connect to WPA/WPA2 network.
      WifiStatus = WiFi.begin(ssid, pass);
      delay(10000);                                         // wait 10 seconds for connection
    }
  }
  else {                                                    // home network
    IPAddress myIP(10, 0, 1, 110 + myIPaddr);               // your desired IP address
    IPAddress gateway(10, 0, 1, 1);                         // your network gateway address
    IPAddress subnet(255, 255, 255, 0);                     // subnet mask
    // attempt to connect to Wifi network:                  // only try if not yet connected
    if ( WiFi.status() != WL_CONNECTED) {
      WiFi.config(myIP, gateway, subnet);                   // Connect to WPA/WPA2 network.
      WifiStatus = WiFi.begin(ssid, pass);
      delay(10000);                                         // wait 10 seconds for connection
    }
  }

  Serial.print("WIFI SSID: ");
  Serial.print(ssid);
  Serial.print(" IP: ");
  Serial.print(WiFi.localIP());
  Serial.print(" port: ");
  Serial.print(IoTPort);
  Serial.print(" WIFI Status: ");
  Serial.println(WifiStatus);

  server.begin();                                           // start web server

  // blink light to indicate startup
  for (int i = 0; i <= 7; i++) {
    digitalWrite(LED, HIGH);                                // turn built-in LED on
    delay(50);
    digitalWrite(LED, LOW);                                 // turn built-in LED off
    delay(50);
  }

  delay(2000);

  // blink light to indicate address
  for (int i = 0; i <= myIPaddr; i++) {
    digitalWrite(LED, HIGH);                                // turn built-in LED on
    delay(250);
    digitalWrite(LED, LOW);                                 // turn built-in LED off
    delay(250);
  }
}




// ----------------------------------------------------------------------------
// Arduino main loop  ---------------------------------------------------------
// ----------------------------------------------------------------------------
void loop() {

  // Serial.print("loop");
  // Check on WiFi staus
  if ( WiFi.status() == WL_CONNECTED) {
    // digitalWrite(LED, HIGH);                             // turn LED on
    // Serial.print(" WiFi ");
  }
  else {
    // digitalWrite(LED, LOW);                              // turn LED off
    // Serial.print(" no WiFi ");
  }
  // Serial.println(" ");

  WiFiClient client = server.available();                   // listen for incoming clients
  if (client) {
    digitalWrite(LED, HIGH);                                // turn request LED on

    // Serial.println("WIFI new client");
    String WebRequest = client.readStringUntil('\r');       // Read the request
    // Serial.println("WIFI Request is '" + WebRequest + "'");

    // send a standard http response header  -----------------------------------
    client.println("HTTP/1.1 200 OK");
    client.println("Content-Type: text/html");
    client.println();
    client.println("<HTML>");
    client.println("<HEAD>");
    client.println("<meta name='viewport' content='width=device-width, initial-scale=1'>");
    client.println("<body bgcolor=AntiqueWhite>");
    client.println("<TITLE>Remote Temperature and Humidity</TITLE>");
    client.println("</HEAD>");
    client.println("<H3>Remote Temperature and Humidity</H3>");
    client.println("<hr width=300 align=LEFT>");
    client.println("<html>");

    // ---------------------------------------------------------------------------
    // web request handling   ----------------------------------------------------
    // ---------------------------------------------------------------------------

    delay(500);
    h = dht.readHumidity();                                 // Read humidity
    f = dht.readTemperature(true);                          // Read temperature as Fahrenheit
    if (isnan(h) || isnan(f)) {                             // Check if any reads failed and try again.
      // Serial.println("Failed to read from DHT sensor");
      while (isnan(h) || isnan(f)) {
        h = dht.readHumidity();                               // Read humidity
        f = dht.readTemperature(true);                        // Read temperature as Fahrenheit
      }
    }
    // Computes temperature values in Fahrenheit and Humidity
    hic = dht.computeHeatIndex(t, h, false);
    dtostrf(hic, 6, 0, celsiusTmpStr);
    hif = dht.computeHeatIndex(f, h);
    dtostrf(hif, 6, 0, fahrenheitTmpStr);
    dtostrf(h, 6, 0, humidityStr);

    // your actual web page that displays temperature and humidity
    client.println("<!DOCTYPE HTML>");
    client.println("<html>");

    client.print("<progress value='");
    client.print(f);
    client.print("' max='100'></progress>");
    client.print("  Temperature: ");
    client.println(fahrenheitTmpStr);
    client.println("&deg;F <br />");

    client.print("<progress value='");
    client.print(h);
    client.print("' max='100'></progress>");
    client.print("  Humidity:    ");
    client.print(humidityStr);
    client.println("% <br />");

    // ---------------------------------------------------------------------------
    // finish up web page  -------------------------------------------------------
    // ---------------------------------------------------------------------------

    client.println("<hr width=300 align=LEFT>");
    client.println(codeVersion);
    client.println("<br />");
    client.println("<hr width=300 align=LEFT>");
    client.println("</BODY>");
    client.println("</HTML>");
    delay(100);                                             // give browser time to receive data
    client.flush();                                         // close the connection
    client.stop();
    // Serial.println("WIFI client disonnected");
  }

  else {
    // Serial.println("WIFI no request found");
  }
  digitalWrite(LED, LOW);                                   // turn request LED off

}

